import { AutocompleteData, NS } from "@ns";
import {parse} from "./lib/flag";
import { installBatchFilePack, isBatchFilePackInstalled, scpBatchFilePack, execWeakenLoop, getRamOfScript } from './lib/batchbase';

/** @param {NS} ns */
export async function main(ns: NS): Promise<void> {
    const flag = parse(ns.args.map(String));
	if(flag._.length < 1){
		ns.tprint("argument required");
		ns.tprint("run cmd [src(hostname)]");
		ns.exit();
	}
	const hostname = String(flag._[0]);

    const th = flag.thread || false;

    ns.scp("share-server.js",hostname);

    let nThread;
    if(th){
        nThread = th;
    }
    else {
        nThread = calculateMaximumThread(ns, hostname, getRamOfScript(ns,"weakenLoop"));
    }
    const pid = ns.exec("share-server.js",hostname, nThread);
	ns.tprint(pid," Process started.");
}

/**
 * @param {NS} ns
 * @param {string} dst
 * @param {number} amount the program ram 
 */
export function calculateMaximumThread(ns:NS,host:string,amount:number):number{
	const maxRam = ns.getServerMaxRam(host);
	const usedRam = ns.getServerUsedRam(host);
	const restRam = maxRam - usedRam;
    return Math.floor(restRam / amount);
}

// eslint-disable-next-line @typescript-eslint/no-unused-vars
export function autocomplete(data: AutocompleteData,args: string[]): string[]{
	return [...data.servers];
}