import { AutocompleteData, NS } from "@ns";
import {parse} from "./lib/flag";
import { installBatchFilePack, isBatchFilePackInstalled, scpBatchFilePack, execWeakenLoop, getRamOfScript } from './lib/batchbase';

/** @param {NS} ns */
export async function main(ns: NS): Promise<void> {
    const flag = parse(ns.args.map(String));
	if(flag._.length < 2){
		ns.tprint("argument required");
		ns.tprint("run cmd [src(hostname)] [dest(hostname)]");
		ns.exit();
	}
	const src = String(flag._[0]);
	const dest = String(flag._[1]);

    const th = flag.thread || false;
    
    if(!isBatchFilePackInstalled(ns)){
        await installBatchFilePack(ns);
    }
    
    await scpBatchFilePack(ns,src);
    
    const stock = flag.stock ?? false;

    let nThread;
    if(th){
        nThread = th;
    }
    else {
        nThread = calculateMaximumThread(ns, src, getRamOfScript(ns,"weakenLoop"));
    }
    const pid = execWeakenLoop(ns,{
        hostname: src,
        target: dest,
        numThread: nThread,
        stock: stock,
    });
	ns.tprint(pid," Process started.");
}

/**
 * @param {NS} ns
 * @param {string} dst
 * @param {number} amount the program ram 
 */
export function calculateMaximumThread(ns:NS,host:string,amount:number):number{
	const maxRam = ns.getServerMaxRam(host);
	const usedRam = ns.getServerUsedRam(host);
	const restRam = maxRam - usedRam;
    return Math.floor(restRam / amount);
}

// eslint-disable-next-line @typescript-eslint/no-unused-vars
export function autocomplete(data: AutocompleteData,args: string[]): string[]{
	return [...data.servers];
}