// 
// You are given the following data, representing a graph:
// [6,[[0,3],[0,4],[4,5],[0,1],[1,2],[1,4]]]
// Note that "graph", as used here, refers to the field of graph theory,
// and has no relation to statistics or plotting. The first element of
// the data represents the number of vertices in the graph. Each vertex 
// is a unique number between 0 and 5. The next element of the data represents
// the edges of the graph. Two vertices u,v in a graph are said to be adjacent
// if there exists an edge [u,v]. Note that an edge [u,v] is the same as
// an edge [v,u], as order does not matter. You must construct a 2-coloring 
// of the graph, meaning that you have to assign each vertex in the graph
// a "color", either 0 or 1, such that no two adjacent vertices have the
// same color. Submit your answer in the form of an array, where element i 
// represents the color of vertex i. If it is impossible to construct a 
// 2-coloring of the given graph, instead submit an empty array.
// 

/**
 * input
 */
 type Input = [number,
    Array<[number,number]>
];

export function graph2coloring([nVertex,edges]:Input):number[]{
    const colors = new Array(nVertex).fill(null);
    const queue = [0];
    colors[0] = false;

    while(queue.length > 0){
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const elem = queue.pop()!;
        const neighbors = getNeighborhood(elem);
        const chcolor = !colors[elem];
        for (const v of neighbors) {
            //propagate color;
            if(colors[v] === null){
                colors[v] = chcolor;
                queue.push(v);
            }
            else if(colors[v] == !chcolor){
                return [];
            }
        }
    }
    return colors.map(x=> x ? 1 : 0);
    function getNeighborhood(i: number): number[]{
        return edges.filter(([a,b])=> a==i || b==i)
            .map(([a,b])=> a == i ? b : a);
    }
}
export function graph2coloringSolve(input:Input):string[]{
    return graph2coloring(input).map(x=>x.toString());
}
export type TestCase = [
    Input,
    number[]
]

export const testcases:TestCase[] = 
[
    [
        [4, [[0, 2], [0, 3], [1, 2], [1, 3]]],
        [0, 0, 1, 1]
    ],
    [
        [3, [[0, 1], [0, 2], [1, 2]]],
        []
    ],
    [
        [6,[[0,3],[0,4],[4,5],[0,1],[1,2],[1,4]]],
        []
    ],
    [
        [9,[[1,4],[3,6],[5,8],[0,1],[7,8],[2,8],[1,8],[2,6],[1,2],[4,6],[4,8]]],
        []
    ]
]

//for (const [data,ans] of testcases) {
//    console.log(graph2coloringSolve(data), ans)
//}