

/**
 * You are located in the top-left corner of the following grid:
 * 
 *   [[0,0,0,0,0,0,0,0,1,0],
 *    [0,0,0,1,0,1,0,0,1,0],
 *    [0,0,0,1,0,0,0,0,0,0],
 *    [0,0,1,0,0,0,0,0,0,1],
 *    [0,0,0,0,1,0,0,0,1,0],
 *    [0,0,1,1,0,1,0,0,0,0],
 *    [1,1,0,0,0,0,0,0,1,0],
 *    [0,0,1,0,0,0,0,0,0,0],
 *    [0,1,1,1,0,0,1,0,0,1],
 *    [0,1,1,0,0,1,0,0,0,0],
 *    [1,0,0,1,0,1,0,0,0,0],
 *    [0,1,0,0,0,0,0,0,0,0]]
 * 
 * You are trying to find the shortest path to the bottom-right
 * corner of the grid, but there are obstacles on the grid that
 * you cannot move onto. These obstacles are denoted by '1', wh
 * ile empty spaces are denoted by 0.
 * 
 * Determine the shortest path from start to finish, if one exi
 * sts. The answer should be given as a string of UDLR characte
 * rs, indicating the moves along the path
 * 
 * NOTE: If there are multiple equally short paths, any of them
 *       is accepted as answer. If there is no path, the answer
 *       should be an empty string.
 * NOTE: The data returned for this contract is an 2D array of 
 *       numbers representing the grid.
 * 
 * Examples:
 * 
 *     [[0,1,0,0,0],
 *      [0,0,0,1,0]]
 * 
 * Answer: 'DRRURRD'
 * 
 *     [[0,1],
 *      [1,0]]
 * 
 * Answer: ''
 */

import { SolveFailError } from "./unsolved";

export function shortestPathInGrid(map: number[][]): string[] {
    const distMap = makeDistanceMap(map, 0, 0);
    if (!isFinite(distMap[map.length - 1][map[map.length - 1].length - 1])) {
        return [];
    }
    const pathes = rewindMap(distMap, map, map[map.length - 1].length - 1, map.length - 1)
    return reversePath(pathes);
}
export function shortestPathInGridSolve(map: number[][]): string[] {
    return [shortestPathInGrid(map).join("")];
}

function copyMap<T>(map: number[][], fillValue: T): T[][] {
    return map.map(x =>
        x.map(_ => fillValue)
    );
}

function makeDistanceMap(map: number[][], initPosX: number, initPosY: number): number[][] {
    const distMap = copyMap(map, Infinity);
    const queue: Array<[number, number]> = [];
    queue.push([initPosX, initPosY]);
    distMap[initPosY][initPosX] = 0;
    while (queue.length > 0) {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        const [x, y] = queue.pop()!;
        const currentDist = distMap[y][x];
        //visit neighbor
        const neighborhood: Array<[number, number]> = [[x + 1, y], [x, y + 1], [x, y - 1], [x - 1, y]]
        for (const [nextX, nextY] of neighborhood) {
            if (isVisitable(map, nextX, nextY)) {
                if (distMap[nextY][nextX] > currentDist + 1) {
                    //set distance
                    distMap[nextY][nextX] = currentDist + 1;
                    //next nodes
                    queue.push([nextX, nextY]);
                }

            }
        }
    }
    return distMap;
}

function isVisitable(map: number[][], x: number, y: number): boolean {
    return 0 <= y && y < map.length && 0 <= x && x < map[y].length && map[y][x] === 0
}

type PathDir = "R" | "D" | "U" | "L";

function rewindMap(distMap: number[][], map: number[][], initPosX: number, initPosY: number): PathDir[] {
    let x = initPosX, y = initPosY;
    const ret: PathDir[] = [];
    while (getMapDist([x, y]) > 0) {
        //walk
        const neighborhood: Array<[number, number]> = [[x + 1, y], [x, y + 1], [x, y - 1], [x - 1, y]]
        const dirNames: PathDir[] = ["R", "D", "U", "L"];
        const candidate = neighborhood.map((x, i) => ({ p: x, index: i }))
            .filter(({ p: [cx, cy] }) => isVisitable(map, cx, cy))
        if (candidate.length === 0) {
            throw new SolveFailError("candidate 0");
        }
        const next = candidate.reduce((v1, v2) => getMapDist(v1.p) < getMapDist(v2.p) ? v1 : v2)
        x = next.p[0];
        y = next.p[1];
        ret.push(dirNames[next.index]);
    }
    return ret;
    function getMapDist([x, y]: [number, number]): number {
        return distMap[y][x];
    }
}


const REVERSE_TABLE: { [key in PathDir]: PathDir } = {
    "R": "L",
    "L": "R",
    "D": "U",
    "U": "D",
}
/**
 * this method mutate array.
 * @param path path to reverse
 * @returns same reference of input array
 */
function reversePath(path: PathDir[]): PathDir[] {
    return path.reverse().map(x => REVERSE_TABLE[x]);
}

 //const map = [
 //    [0, 0, 0, 0, 0, 0, 0, 0, 1, 0],
 //    [0, 0, 0, 1, 0, 1, 0, 0, 1, 0],
 //    [0, 0, 0, 1, 0, 0, 0, 0, 0, 0],
 //    [0, 0, 1, 0, 0, 0, 0, 0, 0, 1],
 //    [0, 0, 0, 0, 1, 0, 0, 0, 1, 0],
 //    [0, 0, 1, 1, 0, 1, 0, 0, 0, 0],
 //    [1, 1, 0, 0, 0, 0, 0, 0, 1, 0],
 //    [0, 0, 1, 0, 0, 0, 0, 0, 0, 0],
 //    [0, 1, 1, 1, 0, 0, 1, 0, 0, 1],
 //    [0, 1, 1, 0, 0, 1, 0, 0, 0, 0],
 //    [1, 0, 0, 1, 0, 1, 0, 0, 0, 0],
 //    [0, 1, 0, 0, 0, 0, 0, 0, 0, 0]
 //]
 //
 //import {sprintf} from "https://deno.land/std@0.152.0/fmt/printf.ts";
 //function sprintDistMap(distMap:number[][]){
 //    return (distMap.map(x=>x.map(y=>isFinite(y)? y : "I")
 //        .map(y=>sprintf("%2s",y.toString())).join(",")).join("\n"));
 //}
 //const distMap = makeDistanceMap(map,0,0);
 //console.log(sprintDistMap(distMap));
 //const pathes = rewindMap(distMap,map,map[map.length - 1].length - 1, map.length - 1)
 //console.log(reversePath(pathes));