import { NS } from '@ns'
import { parse } from './lib/flag';
import { sprintf } from './lib/printf';

interface StockStatsticsInfo{
    stockName: string;
    price: number;
    forecast: number;
    vaolatility: number;
    expectation: number;
    variation: number;
    std:number;
    askPrice:number;
    bidPrice:number;
    maxShare:number;
}

export function getStockStatsticsList(ns:NS): StockStatsticsInfo[]{
    return ns.stock.getSymbols().map(x=>getStockStatstics(ns,x));
}

export function getStockStatstics(ns:NS, stockName:string):StockStatsticsInfo{
    const forecast = ns.stock.getForecast(stockName);
    const val = ns.stock.getVolatility(stockName);
    const price = ns.stock.getPrice(stockName);
    const askPrice = ns.stock.getAskPrice(stockName);
    const bidPrice = ns.stock.getBidPrice(stockName);
    const maxShare = ns.stock.getMaxShares(stockName);
    const priceVar = val * price;
    const expectation = (2*forecast - 1) * priceVar;
    const variation = 4*priceVar*(1-forecast)*forecast;
    return {
        stockName,
        price,
        forecast,
        vaolatility: val,
        expectation: expectation,
        variation: variation,
        std: Math.sqrt(variation),
        askPrice,
        bidPrice,
        maxShare,
    };
}

// eslint-disable-next-line require-await
export async function main(ns: NS) : Promise<void> {
    const flag = parse(ns.args.map(x=>x.toString()));
    if(flag.h || flag.help){
        const msg = ['run cmd']
        msg.forEach(x=>ns.tprint(x));
        return;
    }
    const list = getStockStatsticsList(ns);
    list.sort((a,b)=>b.expectation - a.expectation );
    list.forEach(info=>{
        ns.tprint(sprintf("%6s %8s %2.1f%% %8s %8s",info.stockName,
            ns.nFormat(info.price,"$0.00a"),
            info.forecast*100,
            ns.nFormat(info.expectation,"$0.00a"),
            ns.nFormat(info.std,"$0.00a")))
    })
}