import { selectRootedServerList, ServerInfo } from "lib/servers";
import {
    calculateWeakenTime,
    calculatePercentMoneyHacked,
    calculateHackingExpGain,
    calculateServerGrowth
} from "lib/formula";
import { NS } from '@ns'
import { sprintf } from "./lib/printf";
import { parse } from "./lib/flag";


// eslint-disable-next-line require-await
export async function main(ns: NS): Promise<void> {
    const flag = parse(ns.args.map(x=>x.toString()))

    const list = selectRootedServerList(ns);
    const player = ns.getPlayer();
    const m = list.map(x => {
        const optimalState: ServerInfo = {...x,
            hackDifficulty: x.minDifficulty
        } as ServerInfo;
        ns.print(optimalState.minDifficulty," ", optimalState.hackDifficulty ,"");
        const weakenTime = calculateWeakenTime(optimalState, player);
        const expGain = calculateHackingExpGain(optimalState,player);
        const earnMoney = calculatePercentMoneyHacked(optimalState, player);
        //const growPercent = calculateServerGrowth()
        return {
            hostname: x.hostname,
            info: x,
            weakenTime,
            earnMoney,
            ce: earnMoney* x.moneyMax/weakenTime,
            expCe: expGain / weakenTime,
            expGain,
        }
    });
    if(flag.exp){
        m.sort((a,b)=>(b.expCe-a.expCe));
        
        m.filter(x=>x.expCe > 0).forEach(x=>{
            const msg = sprintf("%20s %8s %6.1fs %8s",x.hostname,
            ns.nFormat(x.expGain,"0.00a"),x.weakenTime,
            ns.nFormat(x.expCe,"0.00a"));
            ns.tprint(msg);
        }) 
    }
    else {
        m.sort((a,b)=>(b.ce-a.ce));
        
        m.filter(x=>x.ce > 0).forEach(x=>{
            const msg = sprintf("%20s %8s %6.1fs %10s",x.hostname,
            ns.nFormat(x.earnMoney * x.info.moneyMax,"$0.00a"),x.weakenTime,
            ns.nFormat(x.ce,"$0.000a"));
            ns.tprint(msg);
        })
    }
}